from enum import EnumMeta


class ArtifactType(EnumMeta):
    """
    Enumeration of supported artifact types.
    """

    MODEL = "MODEL"
    """
    """

    CODE = "CODE"
    """
    """

    DATASET = "DATASET"
    """
    """
