from dataclasses import dataclass
from typing import Optional


@dataclass
class UserVersion:
    description: Optional[str]
    """
    """
    name: Optional[str]
    """
    """
    targetId: Optional[int]
    """
    """
    userVersion: Optional[str]
    """
    """
    uri: Optional[str]
    """
    """
